% This program is for replication of empirical application in 
% "Pooled Bewley Estimator of Long Run Relationships in Dynamic Heterogenous Panels,"
% by A. Chudik, M.H. Pesaran and R. Smith, May 2021.


clear
clc
%% load data first
disp('loading data...')
lpc=load('lpc.dat');
lndi=load('lndi.dat');
dp=load('dp.dat');
disp('... data loaded.')

rng(123456);

missing=8934567;

T = 34;  % 1960 to 1993 
N = 24;  % OECD countries 

% transform variables into T x N matrices, replace missing values with NaNs
Lpc=reshape(lpc,T,N);
Lndi=reshape(lndi,T,N);
Dp=reshape(dp,T,N);

y=Lpc;
x(:,:,1)=Lndi;
x(:,:,2)=Dp;

in=y==missing;
y(in)=NaN;

in=x==missing;
x(in)=NaN;

%% estimations
disp('Computing results...')
tic 
 % pooled Bewley
[betahat, betavar, betastd]=ardlpmg(y,x);
 kappa=diag([1/3;1/3]); [betahatj3, betavarj3, betastdj3]=ardlpmg_vj(y,x, kappa, 1);
 [betahatbc, betavarbc,betavarbc1,kappahat,CVs ]=ardlpmg_bc(y,x);
 kappa=kappahat; [betahatja, betavarja, betastdja]=ardlpmg_vj(y,x, kappa,1);
 %PMG
 [betahatPMG,stdbetaPMG,iter, converge]=PMGe(y,x,1); 
 [PMGresults ]=pmg_bc(y,x); % PMG bias-corrected
 
toc
disp('...computations finished.')
disp('***********************************')

 %% report results
fd='%10.3f'; % reporting precision  

 disp('Pooled Bewley (bias-uncorrected) estimates:')
 disp(num2str(betahat,3));
 disp('95% confidence intervals for pooled Bewley estimates')
 disp(num2str([betahat-1.96*betastd,betahat+1.96*betastd],fd))
 disp('CI length')
 disp(num2str(2*1.96*betastd,fd))
 disp('-----------------------------------')
 
 disp('Simulation-based bias-corrected pooled Bewley estimates:')
 disp(num2str(betahatbc,fd));
 disp('95% confidence intervals for simulation-based bias-corrected pooled Bewley estimates')
 disp(num2str([betahatbc-(betavarbc1.^0.5).*CVs.simulbased',betahatbc+(betavarbc1.^0.5).*CVs.simulbased'],fd))
 disp('CI lenght:')
  disp(num2str(2*(betavarbc1.^0.5).*CVs.simulbased',fd))
 disp('-----------------------------------')

 
 disp('Jackknife bias-corrected pooled Bewley estimates (using kappa=1/3):')
 disp(num2str(betahatj3,fd));
 disp('95% confidence intervals for jackknife bias-corrected pooled Bewley estimates (using kappa=1/3)')
 disp(num2str([betahatj3-betastdj3.*CVs.jack13_jf',betahatj3+betastdj3.*CVs.jack13_jf'],fd))
  disp('CI length')
 disp(num2str(2*betastdj3.*CVs.jack13_jf',fd))
 disp('-----------------------------------')
 
 disp('Jackknife bias-corrected pooled Bewley estimates (using kappahat chosen by simulations):')
 disp(num2str(betahatja,fd));
 disp('95% confidence intervals for jackknife bias-corrected pooled Bewley estimates (using kappahat chosen by simulations)')
 disp(num2str([betahatja-betastdja.*CVs.jackhat_jf',betahatja+betastdja.*CVs.jackhat_jf'],fd))
  disp('CI length')
 disp(num2str(2*betastdja.*CVs.jack13_jf',fd))
 disp('-----------------------------------')
 
 disp('PMG (bias-uncorrected) estimates:')
 disp(num2str(betahatPMG,fd));
 disp('95% confidence intervals for PMG (bias-uncorrected) estimates')
 disp(num2str([betahatPMG-1.96*stdbetaPMG,betahatPMG+1.96*stdbetaPMG],fd))
 disp('CI length')
 disp(num2str(2*1.96*stdbetaPMG,fd))
 disp('-----------------------------------')

 bh=PMGresults.simulbased.betahat;
 bstd=(PMGresults.simulbased.var).^0.5;
 cvb=PMGresults.simulbased.cv;
 disp('Simulation-based bias-corrected PMG estimates:')
 disp(num2str(bh,fd));
 disp('95% confidence intervals for simulation-based bias-corrected PMG estimates')
 disp(num2str([bh-bstd.*cvb',bh+bstd.*cvb'],fd))
 disp('CI length')
 disp(num2str(2*bstd.*cvb',fd))
 disp('-----------------------------------')
 
 bh=PMGresults.jack13.betahat;
 bstd=(PMGresults.jack13.var).^0.5;
 cvb=PMGresults.jack13.cv;
 disp('Jacknife PMG estimates using kappa=1/3:')
 disp(num2str(bh,fd));
 disp('95% confidence intervals for jacknife PMG estimates using kappa=1/3')
 disp(num2str([bh-bstd.*cvb',bh+bstd.*cvb'],fd))
 disp('CI length')
 disp(num2str(2*bstd.*cvb',fd))
 disp('-----------------------------------') 
 
 bh=PMGresults.jackhat.betahat;
 bstd=(PMGresults.jackhat.var).^0.5;
 cvb=PMGresults.jackhat.cv;
 disp('Jacknife PMG estimates using simulated kappahat:')
 disp(num2str(bh,fd));
 disp('95% confidence intervals for jacknife PMG estimates using simulated kappahat')
 disp(num2str([bh-bstd.*cvb',bh+bstd.*cvb'],fd))
 disp('CI length')
 disp(num2str(2*bstd.*cvb',fd))
 disp('-----------------------------------') 
 
 